// This file is generated by kconfig_compiler_kf6 from klines.kcfg.
// All changes you do to this file will be lost.
#ifndef PREFS_H
#define PREFS_H

#include <kconfigskeleton.h>
#include <QCoreApplication>
#include <QDebug>

class Prefs : public KConfigSkeleton
{
  public:

    static Prefs *self();
    ~Prefs() override;

    /**
      Set Whether to show the next set of balls.
    */
    static
    void setShowNext( bool v )
    {
      if (!self()->isShowNextImmutable())
        self()->mShowNext = v;
    }

    /**
      Get Whether to show the next set of balls.
    */
    static
    bool showNext()
    {
      return self()->mShowNext;
    }

    /**
      Is Whether to show the next set of balls. Immutable
    */
    static
    bool isShowNextImmutable()
    {
      return self()->isImmutable( QStringLiteral( "ShowNext" ) );
    }

    /**
      Set Width of saved background
    */
    static
    void setSavedBackgroundWidth( int v )
    {
      if (!self()->isSavedBackgroundWidthImmutable())
        self()->mSavedBackgroundWidth = v;
    }

    /**
      Get Width of saved background
    */
    static
    int savedBackgroundWidth()
    {
      return self()->mSavedBackgroundWidth;
    }

    /**
      Is Width of saved background Immutable
    */
    static
    bool isSavedBackgroundWidthImmutable()
    {
      return self()->isImmutable( QStringLiteral( "SavedBackgroundWidth" ) );
    }

    /**
      Set Height of saved background
    */
    static
    void setSavedBackgroundHeight( int v )
    {
      if (!self()->isSavedBackgroundHeightImmutable())
        self()->mSavedBackgroundHeight = v;
    }

    /**
      Get Height of saved background
    */
    static
    int savedBackgroundHeight()
    {
      return self()->mSavedBackgroundHeight;
    }

    /**
      Is Height of saved background Immutable
    */
    static
    bool isSavedBackgroundHeightImmutable()
    {
      return self()->isImmutable( QStringLiteral( "SavedBackgroundHeight" ) );
    }

  protected:
    Prefs();
    friend class PrefsHelper;


    // Game
    bool mShowNext;
    int mSavedBackgroundWidth;
    int mSavedBackgroundHeight;
};

#endif

